﻿using System;
using static System.Math;
using static System.Console;

namespace DemoCSharp8
{
    public class PatternMatching
    {
        public PatternMatching()
        {

        }

        public void ExtractFromMatching(Shape shape)
        {
            // pattern null
            if (shape is null)
            {
                WriteLine("La forma non è specificata");
                return;
            }

            // pattern type
            if (shape is Rectangle s)
            {
                WriteLine($"La forma è un rettangolo di dimensione {s.Length}x{s.Height}");
                return;
            }
        }

        public void ExtractFromSwitch(Shape shape)
        {
            switch (shape)
            {
                case Circle c:
                    WriteLine($"La forma richiesta è un cerchio di raggio: {c.Radius}");
                    break;

                case Rectangle r when r.Length == r.Height:
                    WriteLine($"La forma è un quadrato di lato {r.Length}");
                    break;

                case Rectangle r:
                    WriteLine($"La forma è un rettangolo di {r.Length}x{r.Height}");
                    break;

                default:
                    WriteLine("La forma non è riconosciuta");
                    break;

                case null:
                    throw new ArgumentNullException(nameof(shape));
            }
        }

        public double GetAreaFromShape(Shape shape) =>
            shape switch
            {
                Rectangle r when r.Length == r.Height => Pow(r.Length, 2),
                Rectangle r => r.Length * r.Height,
                Circle c => Pow(c.Radius, 2) * PI,
                Triangle c => (c.Length * c.Height) / 2,
                _ => 0
            };

        public class Shape
        {
            public int Length { get; set; }
            public int Height { get; set; }
        }

        public class Circle : Shape
        {
            public int Radius { get; set; }
        }

        public class Rectangle : Shape { }
        public class Triangle : Shape { }
    }
}
