﻿using System.Linq;
using System.Collections.Generic;
using static System.Console;

namespace DemoCSharp8
{
    public class PropertyPattern
    {
        public PropertyPattern()
        {
            var itMarket = new Market("IT", .10, .5); // 10%, 5%
            var usMarket = new Market("US", .25, 0);  // 25%, 5%

            var c1 = new Customer("Matteo Tumiati", new Address("IT"));
            var c2 = new Customer("Daniele Bochicchio", new Address("US"));

            var markets = new List<Market> { itMarket, usMarket };
            var orders = new List<Order>
            {
                new Order(c1, itMarket),
                new Order(c1, usMarket),
                new Order(c2, usMarket)
            };

            var amount = orders.Sum(o => o.Amount);
            var amountWithDiscount = CalculateIncome(orders);

            WriteLine($"Il totale degli ordini è {amount}, ma scontato viene {amountWithDiscount}");
        }

        private double CalculateIncome(List<Order> orders)
        {
            foreach (var order in orders)
                order.AmountWithDiscount =
                    CalculateFinalPriceWithDiscount(order.Market, order.Customer.Address, order.Amount);

            return orders.Sum(o => o.AmountWithDiscount);
        }

        private double CalculateFinalPriceWithDiscount(Market market, Address buyFrom, double amount) =>
            (market, buyFrom) switch
            {
                { market: { Name: "IT" }, buyFrom: { State: "IT" } } => amount - (amount * market.DiscountIntern),
                { buyFrom: { State: "US" } } => amount - (amount * market.DiscountIntern),
                _ => amount
            };

        public class Market
        {
            public string Name { get; set; }
            public double DiscountIntern { get; set; }
            public double DiscountForeign { get; set; }

            public Market(string name, double discountIntern, double discountForeign)
            {
                Name = name;
                DiscountIntern = discountIntern;
                DiscountForeign = discountForeign;
            }
        }

        public class Order
        {
            public Customer Customer { get; set; }
            public Market Market { get; set; }
            public double Amount { get; set; } = 100;
            public double AmountWithDiscount { get; set; }

            public Order(Customer customer, Market market)
            {
                Customer = customer;
                Market = market;
            }
        }

        public class Customer
        {
            public string FullName { get; set; }
            public Address Address { get; set; }

            public Customer(string fullName, Address address)
            {
                FullName = fullName;
                Address = address;
            }
        }

        public class Address
        {
            public string State { get; set; }

            public Address(string state)
            {
                State = state;
            }
        }
    }
}